﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Smple_cs_2005
{
	public partial class Form1 : Form
	{
		/// <summary>
		/// アプリケーションのメイン エントリ ポイントです。
		/// </summary>
		[STAThread]
		static void Main()
		{
			Application.Run(new Form1());
		}

		public Form1()
		{
			InitializeComponent();
		}

		/// <summary>
		/// ＦＴＰクライアントクラス
		/// </summary>
		private TKFP.Net.FtpClient FtpClient;
		/// <summary>
		/// カレントディレクトリのクラス
		/// </summary>
		private TKFP.IO.DirectoryInfo CurrentDirectory;

		/// <summary>
		/// 接続開始
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void button1_Click(object sender, System.EventArgs e)
		{
			if (FtpClient != null)
			{
				return;
			}
			//接続用の情報作成
			TKFP.Net.BasicFtpLogon bp = new TKFP.Net.BasicFtpLogon(txtUserID.Text, txtPassword.Text);
			//ＰＯＰクライアントクラスの定義
			FtpClient = new TKFP.Net.FtpClient(bp, txtAddress.Text, 21);
			//パッシブモードに設定
			FtpClient.ConnectionMode = TKFP.Net.ConnectionModes.Passive;
			//リストの取得コマンドを設定
			FtpClient.ListType = TKFP.Net.ListType.LIST;
			//取得したリスト情報の有効時間を設定
			FtpClient.ListCacheValidityInterval = 60;
			FtpClient.FileSystemCacheValidityInterval = 60;

			//FTP over SSL/TLSを使用する場合は以下のように設定します
			//FtpClient.AuthenticationProtocol = TKFP.Net.AuthenticationProtocols.Explicit_TLS;
			//FtpClient.CertificateValidation += new TKFP.Net.CertificateValidationHandler(FtpClient_CertificateValidation);

			//LIST情報の解析クラスを指定
			FtpClient.ListDataLoader = new TKFP.IO.UnixListDataLoader();    //標準のＦＴＰ（デフォルト）
			//FtpClient.ListDataLoader = new TKFP.IO.MsdosListDataLoader ();   //IISでディレクトリ表示スタイルをMS-DOSとした場合

			//日本語ファイル名の文字コードを指定
			//FtpClient.FileNameCharset = "sjis"; //シフトJIS（デフォルト）

			//デバッグ用のログ出力ファイルを指定
			//FtpClient.DebugLogFileName = @"d:\log.txt";


			//モニタ表示のために送受信イベントを登録
			FtpClient.MessageReceive += new TKFP.Net.MessageReceiveHandler(FtpClient_MessageReceive);
			FtpClient.MessageSend += new TKFP.Net.MessageSendHandler(FtpClient_MessageSend);
			//接続開始
			if (!FtpClient.Connect())
			{
				statusBarPanel1.Text = "接続失敗";
				FtpClient.Close();
				FtpClient = null;
				return;
			}
			else
			{
				statusBarPanel1.Text = "接続成功";
				if(FtpClient.IsEncrypted )
				{
	                statusBarPanel1.Text += " (暗号化)";
				}
  
				CurrentDirectory = new TKFP.IO.DirectoryInfo(FtpClient, txtDirectory.Text);
				ShowFolder();

			}
		}


		/// <summary>
		/// サーバーの切断処理
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void button2_Click(object sender, System.EventArgs e)
		{
			if (FtpClient == null)
			{
				return;
			}
			FtpClient.Close();
			FtpClient = null;
			this.lviFolder.Items.Clear();
			statusBarPanel1.Text = "切断";
		}

		/// <summary>
		/// メッセージの受信イベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void FtpClient_MessageReceive(object sender, TKFP.Net.MessageArgs e)
		{
			richTextBox1.SelectionColor = System.Drawing.Color.Blue;
			richTextBox1.AppendText(e.Message);
			richTextBox1.Focus();
			richTextBox1.Refresh();
		}

		/// <summary>
		/// メッセージの送信イベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void FtpClient_MessageSend(object sender, TKFP.Net.MessageArgs e)
		{
			richTextBox1.SelectionColor = System.Drawing.Color.Red;
			richTextBox1.AppendText(e.Message);
			richTextBox1.Focus();
			richTextBox1.Refresh();

		}

		/// <summary>
		/// フォルダの内容を表示する
		/// </summary>
		/// <param name="Path"></param>
		private void ShowFolder()
		{

			FolderName.Text = CurrentDirectory.FullName;
			if (!CurrentDirectory.Parent.FullName.StartsWith(txtDirectory.Text))
			{
				button3.Enabled = false;
			}
			else
			{
				button3.Enabled = true;
			}
			lviFolder.Items.Clear();
			//サブフォルダの一覧取得
			foreach (TKFP.IO.DirectoryInfo d in CurrentDirectory.GetDirectories())
			{
				lviFolder.Items.Add(new DirectoryItem(d));
			}
			//ファイル一覧取得
			foreach (TKFP.IO.FileInfo f in CurrentDirectory.GetFiles())
			{
				lviFolder.Items.Add(new FileItem(f));
			}
		}

		/// <summary>
		/// フォルダの選択処理
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void lviFolder_DoubleClick(object sender, System.EventArgs e)
		{
			if (lviFolder.SelectedItems.Count == 0)
			{
				return;
			}
			System.Windows.Forms.ListViewItem item = lviFolder.SelectedItems[0];

			if (item is DirectoryItem)
			{
				CurrentDirectory = ((DirectoryItem)item).Directory;
				ShowFolder();
			}

		}

		/// <summary>
		/// 親ディレクトリへ移動
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void button3_Click(object sender, System.EventArgs e)
		{
			if (FtpClient == null)
			{
				return;
			}
			CurrentDirectory = CurrentDirectory.Parent;
			ShowFolder();
		}

		/// <summary>
		/// 選択ファイルのダウンロード
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void button4_Click(object sender, System.EventArgs e)
		{
			if (FtpClient == null)
			{
				return;
			}
			if (lviFolder.SelectedItems.Count == 0)
			{
				return;
			}
			System.Windows.Forms.ListViewItem item = lviFolder.SelectedItems[0];

			if (item is FileItem)
			{
				//サーバーファイル情報
				TKFP.IO.FileInfo file = ((FileItem)item).File;
				//デフォルトファイル名
				saveFileDialog1.FileName = file.Name;
				switch (saveFileDialog1.ShowDialog())
				{
					case System.Windows.Forms.DialogResult.OK:
						//ダウンロード
						file.CopyTo(saveFileDialog1.FileName, true);
						break;
				}
			}
		}

		/// <summary>
		/// ファイルのアップロード
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void button5_Click(object sender, System.EventArgs e)
		{
			if (FtpClient == null)
			{
				return;
			}
			switch (openFileDialog1.ShowDialog())
			{
				case System.Windows.Forms.DialogResult.OK:
					//ローカルのファイル情報
					System.IO.FileInfo LocalFile = new System.IO.FileInfo(openFileDialog1.FileName);
					//サーバー用のファイル名作成
					string ServerFile = CurrentDirectory.FullName + LocalFile.Name;
					//サーバー用ファイル情報クラス作成
					TKFP.IO.FileInfo file = new TKFP.IO.FileInfo(CurrentDirectory.FtpClient, ServerFile);
					//アップロード
					file.ReadFrom(LocalFile.FullName);
					//カレントフォルダの情報更新
					CurrentDirectory.Refresh();
					//表示情報更新
					ShowFolder();
					break;
			}
		}

		/// <summary>
		/// SSLの証明書の確認イベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		void FtpClient_CertificateValidation(object sender, TKFP.Net.CertificateValidationArgs e)
		{
			//全ての確認要求を許可します
			e.Cancel = false;
		}


		/// <summary>
		/// ディレクトリ用のListViewItem
		/// </summary>
		private class DirectoryItem : System.Windows.Forms.ListViewItem
		{

			public readonly TKFP.IO.DirectoryInfo Directory;

			public DirectoryItem(TKFP.IO.DirectoryInfo Directory)
			{
				this.Directory = Directory;

				this.Text = Directory.Name;
				this.SubItems.Add("<dir>");
				this.SubItems.Add(Directory.Permission.PermissionCode);
				this.ImageIndex = 0;
			}
		}

		/// <summary>
		/// ファイル用のListViewItem
		/// </summary>
		private class FileItem : System.Windows.Forms.ListViewItem
		{
			public readonly TKFP.IO.FileInfo File;

			public FileItem(TKFP.IO.FileInfo File)
			{
				this.File = File;

				this.Text = File.Name;
				this.SubItems.Add(File.Length.ToString("#,##0"));
				this.SubItems.Add(File.Permission.PermissionCode);
				this.ImageIndex = 1;
			}

		}

	}
}